/*
    Ming, an SWF output library
    Copyright (C) 2001  Opaque Industries - http://www.opaque.net/

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SWF_H_INCLUDED
#define SWF_H_INCLUDED
#include <stdio.h>
#include <stdlib.h>
#include "blocktypes.h"
#include "dll_exports.h"
typedef unsigned char byte;

extern void (*SWF_warn)(char *msg, ...);
extern void (*SWF_error)(char *msg, ...);

WM_DLL_A void WM_DLL_B setSWFWarnFunction(void (*error)(char *msg, ...));
WM_DLL_A void WM_DLL_B setSWFErrorFunction(void (*error)(char *msg, ...));

WM_DLL_A void WM_DLL_B SWF_assert(int c);


  /* a generic output method.  specific instances dump output to file,
     send to stdout, etc. */

typedef void  (*SWFByteOutputMethod)(byte b, void *data);

WM_DLL_A void WM_DLL_B methodWriteUInt16(int i, SWFByteOutputMethod method, void *data);
WM_DLL_A void WM_DLL_B methodWriteUInt32(int i, SWFByteOutputMethod method, void *data);

/* we dig opaque types */


  /* SWFBlock is the parent for all classes in this directory */

typedef void *SWFBlock;
typedef void *SWFOutput;

WM_DLL_A int WM_DLL_B completeSWFBlock(SWFBlock block);
WM_DLL_A void WM_DLL_B destroySWFBlock(SWFBlock block);
WM_DLL_A int WM_DLL_B writeSWFBlockToMethod(SWFBlock block,
			  SWFByteOutputMethod method, void *data);

WM_DLL_A void WM_DLL_B SWFBlock_setDefined(SWFBlock block);
WM_DLL_A byte WM_DLL_B SWFBlock_isDefined(SWFBlock block);

WM_DLL_A SWFBlocktype WM_DLL_B SWFBlock_getType(SWFBlock block);

typedef void (*destroySWFBlockMethod)(struct _block *block);

WM_DLL_A void WM_DLL_B MingUnRecordNewDestructibleBlockRemoved(SWFBlock item);
WM_DLL_A void WM_DLL_B MingRecordNewDestructibleBlockAddedWithDtor(SWFBlock item,destroySWFBlockMethod dtor);
WM_DLL_A void WM_DLL_B MingRecordNewDestructibleBlockAdded(SWFBlock item);
WM_DLL_A void WM_DLL_B MingRemoveAllDestructibleBlocks(void);


  /* SWFInput */

typedef void *SWFInput;

WM_DLL_A int WM_DLL_B SWFInput_length(SWFInput input);
WM_DLL_A void WM_DLL_B SWFInput_rewind(SWFInput input);
WM_DLL_A int WM_DLL_B SWFInput_tell(SWFInput input);
WM_DLL_A void WM_DLL_B SWFInput_seek(SWFInput input, long offset, int whence);
WM_DLL_A int WM_DLL_B SWFInput_eof(SWFInput input);

WM_DLL_A SWFInput WM_DLL_B newSWFInput_file(FILE *f);
WM_DLL_A SWFInput WM_DLL_B newSWFInput_stream(FILE *f);
WM_DLL_A SWFInput WM_DLL_B newSWFInput_buffer(unsigned char *buffer, int length);
WM_DLL_A SWFInput WM_DLL_B newSWFInput_allocedBuffer(unsigned char *buffer, int length);


  /* SWFOutput */

typedef void *SWFOutput;

WM_DLL_A SWFOutput newSWFOutput();
WM_DLL_A SWFOutput WM_DLL_B newSizedSWFOutput(int size);
WM_DLL_A void WM_DLL_B destroySWFOutput(SWFOutput out);

WM_DLL_A int WM_DLL_B SWFOutput_length(SWFOutput out);
WM_DLL_A void WM_DLL_B SWFOutput_byteAlign(SWFOutput out);
WM_DLL_A void WM_DLL_B SWFOutput_writeUInt16(SWFOutput out, int data);
WM_DLL_A void WM_DLL_B SWFOutput_writeToMethod(SWFOutput out,
			     SWFByteOutputMethod method, void *data);


  /* SWFRect */

typedef void *SWFRect;

WM_DLL_A SWFRect WM_DLL_B newSWFRect(int minX, int maxX, int minY, int maxY);
WM_DLL_A void WM_DLL_B destroySWFRect(SWFRect rect);
WM_DLL_A void WM_DLL_B SWFOutput_writeRect(SWFOutput output, SWFRect rect);

WM_DLL_A int WM_DLL_B SWFRect_getWidth(SWFRect r);
WM_DLL_A int WM_DLL_B SWFRect_getHeight(SWFRect r);


  /* SWFCharacter */

/* everything with a character ID is an SWFCharacter */

typedef void *SWFCharacter;

/* this is a silly hack to track blocks which are dependent on others.
   Would be nice to have this in the ming layer above instead */

WM_DLL_A void WM_DLL_B SWFCharacter_addDependency(SWFCharacter character, SWFBlock dependency);
WM_DLL_A SWFBlock *WM_DLL_B SWFCharacter_getDependencies(SWFCharacter character);
WM_DLL_A int WM_DLL_B SWFCharacter_getNDependencies(SWFCharacter character);
WM_DLL_A void WM_DLL_B SWFCharacter_clearDependencies(SWFCharacter character);

WM_DLL_A int WM_DLL_B SWFCharacter_getScaledWidth(SWFCharacter character);
WM_DLL_A int WM_DLL_B SWFCharacter_getScaledHeight(SWFCharacter character);

WM_DLL_A int WM_DLL_B SWFBlock_isCharacter(SWFBlock block);


  /* SWFBitmap */

typedef void *SWFBitmap;

WM_DLL_A void WM_DLL_B destroySWFBitmap(SWFBlock block);
WM_DLL_A int WM_DLL_B SWFBitmap_getWidth(SWFBitmap b);
WM_DLL_A int WM_DLL_B SWFBitmap_getHeight(SWFBitmap b);

WM_DLL_A SWFBitmap WM_DLL_B newSWFBitmap_fromInput(SWFInput input);


  /* SWFDBLBitmap */

typedef void *SWFDBLBitmap;

WM_DLL_A SWFDBLBitmap WM_DLL_B newSWFDBLBitmap(FILE *f);
WM_DLL_A SWFDBLBitmap WM_DLL_B newSWFDBLBitmapForFileName(char *szFileName);
WM_DLL_A SWFDBLBitmap WM_DLL_B newSWFDBLBitmap_fromInput(SWFInput input);


  /* SWFJpegBitmap */

typedef void *SWFJpegBitmap, *SWFJpegWithAlpha;

WM_DLL_A SWFJpegBitmap WM_DLL_B newSWFJpegBitmap(FILE *f);
WM_DLL_A SWFJpegBitmap WM_DLL_B newSWFJpegBitmap_fromInput(SWFInput input);
WM_DLL_A SWFJpegWithAlpha WM_DLL_B newSWFJpegWithAlpha(FILE *f, FILE *alpha);
WM_DLL_A SWFJpegWithAlpha WM_DLL_B newSWFJpegWithAlpha_fromInput(SWFInput input, SWFInput alpha);
WM_DLL_A SWFJpegBitmap WM_DLL_B newSWFJpegBitmapForFileName(char *szJpeg);
WM_DLL_A SWFJpegWithAlpha WM_DLL_B newSWFJpegWithAlphaForFileName(char *szJpeg,char *szAlpha);


  /* SWFGradient */

typedef void *SWFGradient;

WM_DLL_A SWFGradient WM_DLL_B newSWFGradient();
WM_DLL_A void WM_DLL_B destroySWFGradient(SWFGradient gradient);
WM_DLL_A void WM_DLL_B SWFGradient_addEntry(SWFGradient gradient,
			  float ratio, byte r, byte g, byte b, byte a);


  /* SWFMatrix */

typedef void *SWFMatrix;

WM_DLL_A SWFMatrix WM_DLL_B newSWFMatrix(int a, int b, int c, int d, int x, int y);
WM_DLL_A void WM_DLL_B destroySWFMatrix(SWFMatrix matrix);

WM_DLL_A SWFMatrix WM_DLL_B SWFMatrix_set(SWFMatrix m,
			float a, float b, float c, float d, int x, int y);
WM_DLL_A void WM_DLL_B SWFMatrix_clearTranslate(SWFMatrix m);
WM_DLL_A void WM_DLL_B SWFMatrix_clearTransform(SWFMatrix m);


  /* SWFFont */

typedef void *SWFFont;

WM_DLL_A SWFFont WM_DLL_B newSWFFont();
WM_DLL_A SWFFont WM_DLL_B loadSWFFontFromFile(FILE *file);
WM_DLL_A SWFFont WM_DLL_B loadSWFFontFromFileOfName(char *filename);
WM_DLL_A void WM_DLL_B destroySWFFont(SWFBlock block);

WM_DLL_A int WM_DLL_B SWFFont_getScaledStringWidth(SWFFont font, const char *string);

WM_DLL_A short WM_DLL_B SWFFont_getScaledAscent(SWFFont font);
WM_DLL_A short WM_DLL_B SWFFont_getScaledDescent(SWFFont font);
WM_DLL_A short WM_DLL_B SWFFont_getScaledLeading(SWFFont font);


  /* SWFText */

typedef void *SWFText;

WM_DLL_A SWFText WM_DLL_B newSWFText();
WM_DLL_A SWFText WM_DLL_B newSWFText2();
WM_DLL_A void WM_DLL_B destroySWFText(SWFBlock block);

WM_DLL_A void WM_DLL_B SWFText_setFont(SWFText text, SWFFont font);
WM_DLL_A void WM_DLL_B SWFText_setScaledHeight(SWFText text, int height);
WM_DLL_A void WM_DLL_B SWFText_scaledMoveTo(SWFText text, int x, int y);
WM_DLL_A void WM_DLL_B SWFText_setColor(SWFText text, byte r, byte g, byte b, byte a);
WM_DLL_A void WM_DLL_B SWFText_addString(SWFText text, const char *string, int *advance);
WM_DLL_A void WM_DLL_B SWFText_setSpacing(SWFText text, float spacing);

WM_DLL_A int WM_DLL_B SWFText_getScaledStringWidth(SWFText text, const char *string);

WM_DLL_A short WM_DLL_B SWFText_getScaledAscent(SWFText text);
WM_DLL_A short WM_DLL_B SWFText_getScaledDescent(SWFText text);
WM_DLL_A short WM_DLL_B SWFText_getScaledLeading(SWFText text);


/* deprecated: */
#define SWFText_setXY(t,x,y) SWFText_moveTo((t),(x),(y))


  /* SWFBrowserFont */

typedef void *SWFBrowserFont;

WM_DLL_A SWFBrowserFont WM_DLL_B newSWFBrowserFont(char *name);
WM_DLL_A void WM_DLL_B destroySWFBrowserFont(SWFBlock block);


  /* SWFTextField */

typedef void *SWFTextField;

#define SWFTEXTFIELD_ONMASK  0x2085 /* on bits */
#define SWFTEXTFIELD_OFFMASK 0x38FF /* off bits */

#define SWFTEXTFIELD_HASLENGTH (1<<1)
#define SWFTEXTFIELD_NOEDIT    (1<<3)
#define SWFTEXTFIELD_PASSWORD  (1<<4)
#define SWFTEXTFIELD_MULTILINE (1<<5)
#define SWFTEXTFIELD_WORDWRAP  (1<<6)
#define SWFTEXTFIELD_DRAWBOX   (1<<11)
#define SWFTEXTFIELD_NOSELECT  (1<<12)
#define SWFTEXTFIELD_HTML      (1<<9) /* 0x0200 */

typedef enum
{
  SWFTEXTFIELD_ALIGN_LEFT    = 0,
  SWFTEXTFIELD_ALIGN_RIGHT   = 1,
  SWFTEXTFIELD_ALIGN_CENTER  = 2,
  SWFTEXTFIELD_ALIGN_JUSTIFY = 3
} SWFTextFieldAlignment;

WM_DLL_A SWFTextField WM_DLL_B newSWFTextField();
WM_DLL_A void WM_DLL_B destroySWFTextField(SWFBlock block);

WM_DLL_A void WM_DLL_B SWFTextField_setFont(SWFTextField field, SWFBlock font);
WM_DLL_A void WM_DLL_B SWFTextField_setScaledBounds(SWFTextField field, int width, int height);
WM_DLL_A void WM_DLL_B SWFTextField_setFlags(SWFTextField field, int flags);
WM_DLL_A void WM_DLL_B SWFTextField_setColor(SWFTextField field, byte r, byte g, byte b, byte a);
WM_DLL_A void WM_DLL_B SWFTextField_setVariableName(SWFTextField field, char *name);
WM_DLL_A void WM_DLL_B SWFTextField_addString(SWFTextField field, char *string);

WM_DLL_A void WM_DLL_B SWFTextField_setScaledFontHeight(SWFTextField field, int height);
WM_DLL_A void WM_DLL_B SWFTextField_setScaledFieldHeight(SWFTextField field, int height);
WM_DLL_A void WM_DLL_B SWFTextField_setScaledLeftMargin(SWFTextField field, int leftMargin);
WM_DLL_A void WM_DLL_B SWFTextField_setScaledRightMargin(SWFTextField field, int rightMargin);
WM_DLL_A void WM_DLL_B SWFTextField_setScaledIndentation(SWFTextField field, int indentation);
WM_DLL_A void WM_DLL_B SWFTextField_setScaledLineSpacing(SWFTextField field, int lineSpacing);
WM_DLL_A void WM_DLL_B SWFTextField_setAlignment(SWFTextField field,
			       SWFTextFieldAlignment alignment);
WM_DLL_A void WM_DLL_B SWFTextField_setLength(SWFTextField field, int length);


  /* SWFFillStyle */

typedef void *SWFFillStyle;

#define SWFFILL_SOLID		0x00
#define SWFFILL_GRADIENT	0x10
#define SWFFILL_LINEAR_GRADIENT 0x10
#define SWFFILL_RADIAL_GRADIENT 0x12
#define SWFFILL_BITMAP		0x40
#define SWFFILL_TILED_BITMAP	0x40
#define SWFFILL_CLIPPED_BITMAP	0x41

WM_DLL_A SWFFillStyle WM_DLL_B newSWFSolidFillStyle(byte r, byte g, byte b, byte a);
WM_DLL_A SWFFillStyle WM_DLL_B newSWFGradientFillStyle(SWFGradient gradient, byte radial);
WM_DLL_A SWFFillStyle WM_DLL_B newSWFBitmapFillStyle(SWFCharacter bitmap, byte flags);

WM_DLL_A SWFMatrix WM_DLL_B SWFFillStyle_getMatrix(SWFFillStyle fill);


  /* SWFLineStyle */

typedef void *SWFLineStyle;

WM_DLL_A SWFLineStyle WM_DLL_B newSWFLineStyle(int width, int r, int g, int b, int a);
WM_DLL_A byte WM_DLL_B SWFLineStyle_equals(SWFLineStyle line, unsigned short width,
			 byte r, byte g, byte b, byte a);


  /* SWFShape */

typedef void *SWFShape;

WM_DLL_A SWFShape WM_DLL_B newSWFShape();
WM_DLL_A void WM_DLL_B destroySWFShape(SWFBlock block);

WM_DLL_A int WM_DLL_B SWFShape_getScaledPenX(SWFShape shape);
WM_DLL_A int WM_DLL_B SWFShape_getScaledPenY(SWFShape shape);

WM_DLL_A void WM_DLL_B SWFShape_moveScaledPenTo(SWFShape shape, int x, int y);
WM_DLL_A void WM_DLL_B SWFShape_moveScaledPen(SWFShape shape, int x, int y);

WM_DLL_A void WM_DLL_B SWFShape_drawScaledLineTo(SWFShape shape, int x, int y);
WM_DLL_A void WM_DLL_B SWFShape_drawScaledLine(SWFShape shape, int dx, int dy);
WM_DLL_A void WM_DLL_B SWFShape_drawScaledCurveTo(SWFShape shape, int controlx, int controly,
				int anchorx, int anchory);
WM_DLL_A void WM_DLL_B SWFShape_drawScaledCurve(SWFShape shape, int controldx, int controldy,
			      int anchordx, int anchordy);

WM_DLL_A void WM_DLL_B SWFShape_drawScaledGlyph(SWFShape shape,
			      SWFFont font, unsigned char c, int size);


/* deprecated: */

#define SWFShape_moveTo SWFShape_moveScaledPenTo
#define SWFShape_moveToRelative SWFShape_moveScaledPen
#define SWFShape_lineTo SWFShape_drawScaledLineTo
#define SWFShape_lineToRelative SWFShape_drawScaledLine
#define SWFShape_curveTo SWFShape_drawScaledCurveTo
#define SWFShape_curveToRelative SWFShape_drawScaledCurve

WM_DLL_A void WM_DLL_B SWFShape_end(SWFShape shape);

WM_DLL_A int WM_DLL_B SWFShape_setLineStyle(SWFShape shape, unsigned short width,
			  byte r, byte g, byte b, byte a);

WM_DLL_A SWFFillStyle WM_DLL_B SWFShape_addSolidFillStyle(SWFShape shape,
					byte r, byte g, byte b, byte a);

WM_DLL_A SWFFillStyle WM_DLL_B SWFShape_addGradientFillStyle(SWFShape shape,
					   SWFGradient gradient, byte flags);

WM_DLL_A SWFFillStyle WM_DLL_B SWFShape_addBitmapFillStyle(SWFShape shape,
					 SWFBitmap bitmap, byte flags);

WM_DLL_A void WM_DLL_B SWFShape_setLeftFillStyle(SWFShape shape, SWFFillStyle fill);
WM_DLL_A void WM_DLL_B SWFShape_setRightFillStyle(SWFShape shape, SWFFillStyle fill);


  /* SWFMorph */

typedef void *SWFMorph;

WM_DLL_A SWFMorph WM_DLL_B newSWFMorphShape();
WM_DLL_A void WM_DLL_B destroySWFMorph(SWFBlock block);
WM_DLL_A SWFShape WM_DLL_B SWFMorph_getShape1(SWFMorph morph);
WM_DLL_A SWFShape WM_DLL_B SWFMorph_getShape2(SWFMorph morph);


  /* sound - only mp3 streaming implemented */

typedef void *SWFSound;

#define SWF_SOUND_COMPRESSION      0xf0
#define SWF_SOUND_NOT_COMPRESSED   (0<<4)
#define SWF_SOUND_ADPCM_COMPRESSED (1<<4)

#define SWF_SOUND_RATE             0x0c
#define SWF_SOUND_5KHZ             (0<<2)
#define SWF_SOUND_11KHZ            (1<<2)
#define SWF_SOUND_22KHZ            (2<<2)
#define SWF_SOUND_44KHZ            (3<<2)

#define SWF_SOUND_BITS             0x02
#define SWF_SOUND_8BIT             (0<<1)
#define SWF_SOUND_16BIT            (1<<1)

#define SWF_SOUND_CHANNELS         0x01
#define SWF_SOUND_MONO             (0<<0)
#define SWF_SOUND_STEREO           (1<<0)

WM_DLL_A SWFSound WM_DLL_B newSWFSound(FILE *file);
WM_DLL_A SWFSound WM_DLL_B newSWFSoundForFileName(char *szname);
WM_DLL_A SWFSound WM_DLL_B newSWFSound_fromInput(SWFInput input);
WM_DLL_A void WM_DLL_B destroySWFSound(SWFSound sound);

WM_DLL_A SWFBlock WM_DLL_B SWFSound_getStreamHead(SWFSound sound, float frameRate);
WM_DLL_A SWFBlock WM_DLL_B SWFSound_getStreamBlock(SWFSound sound);
WM_DLL_A void WM_DLL_B SWFSound_rewind(SWFSound sound);


  /* SWFCXform */

typedef void *SWFCXform;

WM_DLL_A SWFCXform WM_DLL_B newSWFCXform(int rAdd, int gAdd, int bAdd, int aAdd,
		       float rMult, float gMult, float bMult, float aMult);

WM_DLL_A SWFCXform WM_DLL_B newSWFAddCXform(int rAdd, int gAdd, int bAdd, int aAdd);

WM_DLL_A SWFCXform WM_DLL_B newSWFMultCXform(float rMult, float gMult, float bMult, float aMult);

WM_DLL_A void WM_DLL_B SWFCXform_setColorAdd(SWFCXform cXform,
			   int rAdd, int gAdd, int bAdd, int aAdd);

WM_DLL_A void WM_DLL_B SWFCXform_setColorMult(SWFCXform cXform,
			    float rMult, float gMult, float bMult, float aMult);

WM_DLL_A void WM_DLL_B destroySWFCXform(SWFCXform cXform);


  /* SWFAction */

typedef void *SWFAction;

WM_DLL_A SWFAction WM_DLL_B newSWFAction();
WM_DLL_A SWFAction WM_DLL_B newSWFAction_fromOutput(SWFOutput out);
WM_DLL_A SWFAction WM_DLL_B compileSWFActionCode(char *script);
WM_DLL_A void WM_DLL_B destroySWFAction(SWFAction action);


  /* placeobject.h */

typedef void *SWFPlaceObject2Block;

#define SWF_PLACEACTION_ONLOAD      (1<<0)
#define SWF_PLACEACTION_ENTERFRAME  (1<<1)
#define SWF_PLACEACTION_UNLOAD      (1<<2)
#define SWF_PLACEACTION_MOUSEMOVE   (1<<3)
#define SWF_PLACEACTION_MOUSEDOWN   (1<<4)
#define SWF_PLACEACTION_MOUSEUP     (1<<5)
#define SWF_PLACEACTION_KEYDOWN     (1<<6)
#define SWF_PLACEACTION_KEYUP       (1<<7)
#define SWF_PLACEACTION_DATA        (1<<8)

WM_DLL_A SWFPlaceObject2Block WM_DLL_B newSWFPlaceObject2Block();

WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setDepth(SWFPlaceObject2Block block, int depth);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setName(SWFPlaceObject2Block block,
				  const char *name);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setRatio(SWFPlaceObject2Block block, int ratio);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setMaskLevel(SWFPlaceObject2Block block,
				       int masklevel);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setCXform(SWFPlaceObject2Block block,
				    SWFCXform cXform);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setColorAdd(SWFPlaceObject2Block block,
				      int r, int g, int b, int a);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setColorMult(SWFPlaceObject2Block block,
				       float r, float g, float b, float a);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setMatrix(SWFPlaceObject2Block block,
				    SWFMatrix matrix);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setCharacter(SWFPlaceObject2Block block,
				       SWFCharacter character);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_setMove(SWFPlaceObject2Block block);
WM_DLL_A void WM_DLL_B SWFPlaceObject2Block_addAction(SWFPlaceObject2Block block,
				    SWFAction action, int flags);


  /* random blocks */

typedef void *SWFOutputBlock;
typedef void *SWFExports;

WM_DLL_A SWFBlock WM_DLL_B newSWFPlaceObjectBlock(SWFCharacter character, int depth,
				SWFMatrix matrix, SWFCXform cXform);

WM_DLL_A SWFOutputBlock WM_DLL_B newSWFRemoveObjectBlock(SWFCharacter character, int depth);
WM_DLL_A SWFOutputBlock WM_DLL_B newSWFRemoveObject2Block(int depth);
WM_DLL_A SWFOutputBlock WM_DLL_B newSWFFrameLabelBlock(char *string);
WM_DLL_A SWFBlock WM_DLL_B newSWFSetBackgroundBlock(byte r, byte g, byte b);
WM_DLL_A SWFBlock WM_DLL_B newSWFShowFrameBlock();
WM_DLL_A SWFBlock WM_DLL_B newSWFEndBlock();
WM_DLL_A SWFBlock WM_DLL_B newSWFProtectBlock();
WM_DLL_A SWFOutputBlock WM_DLL_B newSWFExportBlock(SWFExports exports, int nExports);


  /* SWFButton */

typedef void *SWFButton;

#define SWFBUTTON_HIT    (1<<3)
#define SWFBUTTON_DOWN   (1<<2)
#define SWFBUTTON_OVER   (1<<1)
#define SWFBUTTON_UP     (1<<0)

/* deprecated: */

  #define SWFBUTTONRECORD_HITSTATE    (1<<3)
  #define SWFBUTTONRECORD_DOWNSTATE   (1<<2)
  #define SWFBUTTONRECORD_OVERSTATE   (1<<1)
  #define SWFBUTTONRECORD_UPSTATE     (1<<0)


#define SWFBUTTON_KEYPRESS(c)     (((c)&0x7f)<<9)
#define SWFBUTTON_ONKEYPRESS(c)     (((c)&0x7f)<<9)

#define SWFBUTTON_OVERDOWNTOIDLE    (1<<8)
#define SWFBUTTON_IDLETOOVERDOWN    (1<<7)
#define SWFBUTTON_OUTDOWNTOIDLE     (1<<6)
#define SWFBUTTON_OUTDOWNTOOVERDOWN (1<<5)
#define SWFBUTTON_OVERDOWNTOOUTDOWN (1<<4)
#define SWFBUTTON_OVERDOWNTOOVERUP  (1<<3)
#define SWFBUTTON_OVERUPTOOVERDOWN  (1<<2)
#define SWFBUTTON_OVERUPTOIDLE      (1<<1)
#define SWFBUTTON_IDLETOOVERUP      (1<<0)

/* easier to remember: */
#define SWFBUTTON_MOUSEUPOUTSIDE  SWFBUTTON_OUTDOWNTOIDLE
#define SWFBUTTON_DRAGOVER        (SWFBUTTON_OUTDOWNTOOVERDOWN | SWFBUTTON_IDLETOOVERDOWN)
#define SWFBUTTON_DRAGOUT         (SWFBUTTON_OVERDOWNTOOUTDOWN | SWFBUTTON_OVERDOWNTOIDLE)
#define SWFBUTTON_MOUSEUP         SWFBUTTON_OVERDOWNTOOVERUP
#define SWFBUTTON_MOUSEDOWN       SWFBUTTON_OVERUPTOOVERDOWN
#define SWFBUTTON_MOUSEOUT        SWFBUTTON_OVERUPTOIDLE
#define SWFBUTTON_MOUSEOVER       SWFBUTTON_IDLETOOVERUP


WM_DLL_A SWFButton WM_DLL_B newSWFButton();
WM_DLL_A void WM_DLL_B destroySWFButton(SWFBlock block);

WM_DLL_A void WM_DLL_B SWFButton_addShape(SWFButton button, SWFCharacter character, byte flags);
WM_DLL_A void WM_DLL_B SWFButton_addAction(SWFButton button, SWFAction action, int flags);

WM_DLL_A SWFBlock WM_DLL_B newSWFButtonCXform(SWFButton button, SWFCXform *cXforms);
WM_DLL_A SWFBlock WM_DLL_B newSWFButtonSound(SWFButton button, SWFSound sound);


  /* SWFSprite */

typedef void *SWFSprite;

WM_DLL_A SWFSprite WM_DLL_B newSWFSprite();
WM_DLL_A void WM_DLL_B destroySWFSprite(SWFBlock block);

WM_DLL_A void WM_DLL_B SWFSprite_setNumberOfFrames(SWFSprite sprite, int totalFrames);

WM_DLL_A void WM_DLL_B SWFSprite_setBlocks(SWFSprite sprite, SWFBlock *blocks, int nBlocks);
WM_DLL_A void WM_DLL_B SWFSprite_addBlock(SWFSprite sprite, SWFBlock block);

// buffered file output
typedef void *SFILE;
WM_DLL_A void WM_DLL_B fclose_Buf(SFILE *f);
WM_DLL_A SFILE * WM_DLL_B fopen_Buf(char *szFile,char *szmethod);

typedef void *(*SWFFileOpenMethod)(char *szFile,const char *mode);
typedef void (*SWFFileCloseMethod)(void *data);
void fileOutputMethod(byte b, void *data);

WM_DLL_A void WM_DLL_B Set_fileByteOutputMethod(SWFByteOutputMethod new_method);
WM_DLL_A void WM_DLL_B Set_fileOpenMethod(SWFFileOpenMethod new_method);
WM_DLL_A void WM_DLL_B Set_fileCloseMethod(SWFFileCloseMethod new_method);

// win_ming
#ifdef cl_win
#include <windows.h>
typedef struct _swfMovie *SWFMovie;
typedef struct _swfDisplayItem *SWFDisplayItem;
WM_DLL_A void WM_DLL_B WinMing_SelectMovie(HDC hdc,SWFMovie m);
WM_DLL_A void WM_DLL_B WinMing_UnSelectMovie(HDC hdc,SWFMovie m);
WM_DLL_A SWFMovie WM_DLL_B WinMing_GetMovieForHdc(HDC hdc);
WM_DLL_A void WM_DLL_B WinMing_FreeMovieSels(void);
WM_DLL_A void WM_DLL_B WinMing_FreeMovieSelsIfPoss(char free_anyway);
WM_DLL_A SWFMovie WM_DLL_B WinMing_StartMovie(int width,int height
                                        ,float frames_per_sec,int nframes);
WM_DLL_A int WM_DLL_B WinMing_CloseMovie(SWFMovie m,char *szFileName);
WM_DLL_A SWFDisplayItem WM_DLL_B WinMing_Polygon(HDC hdc,POINT *pts,int npts);
WM_DLL_A SWFDisplayItem WM_DLL_B WinMing_Rectangle(HDC hdc,int left,int top,int right,int bottom);
WM_DLL_A SWFDisplayItem WM_DLL_B WinMing_Polyline(HDC hdc,POINT *pts,int npts);
WM_DLL_A SWFDisplayItem WM_DLL_B WinMing_PolyPolygon(HDC hdc,POINT *poly_pts,int *poly_counts,int ncounts);
WM_DLL_A SWFDisplayItem WM_DLL_B WinMing_LineTo(HDC hdc,int x,int y);
WM_DLL_A BOOL WM_DLL_B WinMing_MoveTo(HDC hdc,int x,int y,POINT *pt);
WM_DLL_A void WM_DLL_B WinMing_FindBrushRGB(HBRUSH hBrush,BYTE *pR,BYTE *pG,BYTE *pB);
WM_DLL_A void WM_DLL_B WinMing_FindPenWidthAndRGB(HPEN hPen,int *pwidth,BYTE *pR,BYTE *pG,BYTE *pB);
WM_DLL_A SWFDisplayItem WM_DLL_B WinMing_GradientFillRect(HDC hdc
               ,int degrees
               ,int left,int top,int right,int bottom
               ,byte rfill, byte gfill, byte bfill, byte afill
               ,byte rfill2, byte gfill2, byte bfill2, byte afill2
               );
WM_DLL_A void WM_DLL_B WinMing_SetGDIMaskLevel(HDC hdc,int ilevel);
WM_DLL_A void WM_DLL_B WinMing_SetGDIMaskAsWhite(HDC hdc,BOOL b);
WM_DLL_A void WM_DLL_B WinMing_StartPaintMask(HDC hdc);
WM_DLL_A void WM_DLL_B WinMing_StopPaintMask(HDC hdc);
typedef struct swfJpeg *SWFJpeg;
WM_DLL_A SWFJpeg WM_DLL_B WinMing_InitJpegFillFromFileName(HDC hdc,char *szJpegFileName);
WM_DLL_A void WM_DLL_B WinMing_InitJpegFill(HDC hdc,SWFJpeg jpeg);
WM_DLL_A void WM_DLL_B WinMing_StartJpegFill(HDC hdc);
WM_DLL_A void WM_DLL_B WinMing_StopJpegFill(HDC hdc);
// note you can only have one jpeg fill on the go at a time
// if you want to reuse them then keep track of the
// pointers returned by InitJpegFillFromFileName
// and use them for InitJpegFill
WM_DLL_A HBRUSH WM_DLL_B WinMing_FindCurrentBrushRGB(HDC hdc,BYTE *pR,BYTE *pG,BYTE *pB);
WM_DLL_A HPEN WM_DLL_B WinMing_FindCurrentPenWidthAndRGB(HDC hdc,int *pwidth,BYTE *pR,BYTE *pG,BYTE *pB);
WM_DLL_A void WM_DLL_B WinMing_SetBackground(HDC hdc,HBRUSH hBrush);

// when movie is finished
// and output, call this to remove all the shapes
// and fills and gradients you made for it.
// - Called in Ming_clean_up()
WM_DLL_A void WM_DLL_B MingRemoveAllDestructibleBlocks(void);
WM_DLL_A void WM_DLL_B MingRemoveAllDisplayItems(char remove_forgotten_items);
WM_DLL_A void WM_DLL_B MingRemoveAllFills(void);
WM_DLL_A void WM_DLL_B MingRemoveAllGradients(void);
WM_DLL_A void WM_DLL_B MingCloseAllFiles(void);
WM_DLL_A void WM_DLL_B FreeMovieSels(void);

// Can also use MingRemoveAllDisplayItems(0)
// at end of each frame if you want a clean slate
// but if you want to keep one of the items, e.g. a bitmap filled square
// call this with return value from SWFMovie_add_item()
typedef struct _swfDisplayItem *SWFDisplayItem;
WM_DLL_A void WM_DLL_B MingForgetDisplayItem(SWFDisplayItem item);
WM_DLL_A void WM_DLL_B MingRememberAllDisplayItems(SWFDisplayItem item);


WM_DLL_A int WM_DLL_B MingFreeAllPointersAllocated(void);

// for typical use of the win_ming gdi calls see win_ming_dll.h

#endif

#endif /* SWF_H_INCLUDED */


